#!/bin/bash

##################################################################################################################
#                                                                                                                #
#                       daccord Microsoft Edition Server v.2.1.x - Installation script                           #
#                                                                                                                #
##################################################################################################################

# Set stage to either "DEVELOPMENT" or "PRODUCTION" to influence what repository the rpm files are pulled from
STAGE="PRODUCTION"
# Timestamp variable used to timestamp log entries
NOW=$(date +"%d-%m-%Y %T")
# Date variable used to give the logfile a name including the current date + hour & minute of creation
DATE=$(date +"%Y-%m-%d-%H-%M-%S")
# Logfile variable that holds the name of the logifle including the $DATE variable from above
LOGFILE=daccord-mse-install-${DATE}.log

# If the user id isn't 0, the script isn't being run as root and gives out a warning that it needs to be run as root to run successfully
CHECK_ROOTPRIVILEGE(){
    if [[ $EUID -ne 0 ]]; then
        echo -e "\033[0;31mThis script must be run with root privileges! Please verify that you have the needed privileges and restart the installation!\033[m"
        EXIT
    fi
}

# Check if the script is running on openSUSE or SLES or Red Hat
CHECK_OS(){
    cat /etc/os-release >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        OS=$(cat /etc/os-release | grep -e "^NAME=")
        if [[ ${OS} == 'NAME="openSUSE Leap"' ]] || [[ ${OS} == 'NAME="SLES"' ]]; then
            REDHAT="FALSE"
        else
            REDHAT="TRUE"
        fi
    else
        REDHAT="TRUE"
    fi
}

# Check if the stage variable is set to "DEVELOPMENT" and warn about it if set as such
CHECK_DEV(){
    if [[ $STAGE == "DEVELOPMENT" ]]; then
        echo -e "\033[0;31m\nWARNING! - This script is running in DEVELOPMENT mode.\033[m"
        echo -e "\033[0;31mIt will install or update from DEVELOPMENT repositories.\033[m"
    fi
}

# Creates a file to let the script notice on the next run if the previous installation didn't run through entirely
SET_PREVINSTALL_FLAG(){
    mkdir -p /opt/GuH/ >> ./${LOGFILE} 2>&1
    touch /opt/GuH/daccord_mse_install_flag >> ./${LOGFILE} 2>&1
}

# Check if the file that is being created by the above funtion exists on this system
CHECK_PREVINSTALL_FLAG(){
    if [[ -e /opt/GuH/daccord_mse_install_flag ]]; then
        PREVINSTALL=1
    fi
}

CANCEL(){
    echo "Canceling the install script!"
    echo "[${NOW} install.sh] Script has been canceled." >> ./${LOGFILE}
    exit 1
}

EXIT(){
    exit 1
}

PRESS_ANY_KEY(){
    echo -e "Press any key to continue."
    read -sn 1
}

# Check if the system currently already has the daccord server software installed
CHECK_INST_CURRENT(){
    DACCORD_CURRENT_STAT=-1
    if [[ -e /opt/GuH/inchorus-portal-gadget ]]; then
        DACCORD_CURRENT_STAT=1
    fi
}


OUTPUT_WELCOME(){
    clear
    echo -e "\033[0;36m################################################################################\033[m"
    echo -e "\033[0;36m#                                                                              #\033[m"
    echo -e "\033[0;36m#\033[m  \033[mdaccord Microsoft Edition Server v.2.1.x - Installation script\033[m              \033[0;36m#\033[m"
    echo -e "\033[0;36m#                                                                              #\033[m"
    echo -e "\033[0;36m################################################################################\033[m"
    echo -e " "
    echo -e "For detailed information please refer to the logfile: ${PWD}/${LOGFILE}"
}

# Give the user the option to install mariadb on a SUSE system through zypper (online modus) if not already installed
CHECK_MYSQL_SUSE(){
    rpm -qa | grep -i '^mariadb-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        echo -e "\nA mariadb server is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] A mariadb server is already installed on the server." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\nThe daccord Microsoft Edition requires a mariadb server to store data."
            read -p $'\033[0;36mDo you want to install a local [l] mariadb server or use an existing external [e] mariadb server? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install a local mariadb server or [e] to use an existing external mariadb server!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nPlease wait while the mariadb server is being installed."
            zypper --non-interactive in mariadb >> ./${LOGFILE} 2>&1
            while true; do
                rpm -qa | grep -i '^mariadb-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    sleep 2
                    break
                fi
                grep "Failed dependencies" >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    break
                fi
                echo -n '.'
                sleep 1
            done
            systemctl start mariadb.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The local mariadb server has been successfully installed." >> ./${LOGFILE}
                echo -e "\nThe local mariadb server has been successfully installed."
                echo -e "\nPlease wait while the mariadb server is being prepared. This may take several minutes."

                rpm -qa | grep -i '^expect-[0-9]' >> /dev/null 2>&1
                if [[ $? != 0 ]]; then
                    zypper --non-interactive in expect >> ./${LOGFILE} 2>&1
                fi

                MYSQLPW=
                MYSQL_ROOT_PASSWORD=daccord
                SECURE_MYSQL=$(expect -c "

                set timeout 10
                spawn mysql_secure_installation

                expect \"Enter current password for root (enter for none):\"
                send \"${MYSQLPW}\r\"

                expect \"Change the root password?\"
                send \"y\r\"

                expect \"New password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Re-enter new password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Remove anonymous users?\"
                send \"y\r\"

                expect \"Disallow root login remotely?\"
                send \"y\r\"

                expect \"Remove test database and access to it?\"
                send \"y\r\"

                expect \"Reload privilege tables now?\"
                send \"y\r\"

                expect eof
                ")

                mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                while [[ ${mariadb_check} = "" ]]; do
                    echo -n '.'
                    sleep 1
                    mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                done

                echo "${SECURE_MYSQL}" >> /dev/null &

                MYSQL_USER=`which mysql`

                sed -i s/bind-address/#bind-address/g /etc/my.cnf >> /dev/null 2>&1

                Q1="CREATE USER daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C';"
                Q2="CREATE DATABASE daccord;"
                Q3="CREATE DATABASE inchorus;"
                Q4="GRANT ALL PRIVILEGES ON *.* TO daccord@'%' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q5="GRANT ALL PRIVILEGES ON *.* TO daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q6="GRANT ALL PRIVILEGES ON *.* TO root@'%' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q7="GRANT ALL PRIVILEGES ON *.* TO root@'localhost' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q8="FLUSH PRIVILEGES;"
                SQL="${Q1}${Q2}${Q3}${Q4}${Q5}${Q6}${Q7}${Q8}"
                ${MYSQL_USER} -uroot -p${MYSQL_ROOT_PASSWORD} -e "${SQL}" &

                systemctl enable mariadb.service >> ./${LOGFILE} 2>&1

                echo -e "\nThe mariadb server has been successfully prepared. Please note that a user (daccord) with the password (daccord) has been created."
            else
                echo "[${NOW} install.sh] An error occured while trying to install the mariadb server through zypper." >> ./${LOGFILE}
                echo -e "\nAn error occured while trying to install the mariadb server through zypper, please view the ${LOGFILE} file in the directory where the script file is located."
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nYou have chosen not to install a local mariadb server."
            echo -e "\nPlease have your login credentials on hand for the configuration at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

# Function that gives the user the option to install mariadb on a centOS system through yum (online modus) if not already installed
CHECK_MYSQL_REDHAT(){
    rpm -qa | grep -i '^mariadb-server-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        echo -e "\nA mariadb server is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] A mariadb server is already installed on the server." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\nThe daccord Microsoft Edition requires a mariadb server to store data."
            read -p $'\033[0;36mDo you want to install a local [l] mariadb server or use an existing external [e] mariadb server? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install a local mariadb server or [e] to use an existing external mariadb server!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nPlease wait while the mariadb server is being installed."
            yum install -y mariadb-server >> ./${LOGFILE} 2>&1
            while true; do
                rpm -qa | grep mariadb-server >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    sleep 15
                    break
                fi
                grep "Failed dependencies" >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    break
                fi
                echo -n '.'
                sleep 1
            done
            systemctl start mariadb.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The local mariadb server has been successfully installed." >> ./${LOGFILE}
                echo -e "\nThe local mariadb server has been successfully installed."
                echo -e "\nPlease wait while the mariadb server is being prepared. This may take several minutes."

                rpm -qa | grep -i '^expect-[0-9]' >> /dev/null 2>&1
                if [[ $? != 0 ]]; then
                    yum install -y expect >> ./${LOGFILE} 2>&1
                fi

                MYSQLPW=""
                MYSQL_ROOT_PASSWORD="daccord"
                SECURE_MYSQL=$(expect -c "

                set timeout 10
                spawn mysql_secure_installation

                expect \"Enter current password for root (enter for none):\"
                send \"${MYSQLPW}\r\"

                expect \"Change the root password?\"
                send \"y\r\"

                expect \"New password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Re-enter new password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Remove anonymous users?\"
                send \"y\r\"

                expect \"Disallow root login remotely?\"
                send \"y\r\"

                expect \"Remove test database and access to it?\"
                send \"y\r\"

                expect \"Reload privilege tables now?\"
                send \"y\r\"

                expect eof
                ")

                mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                while [[ ${mariadb_check} = "" ]]; do
                    echo -n '.'
                    sleep 1
                    mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                done

                echo "${SECURE_MYSQL}" >> /dev/null &

                MYSQL_USER=`which mysql`

                Q1="CREATE USER daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C';"
                Q2="CREATE DATABASE daccord;"
                Q3="CREATE DATABASE inchorus;"
                Q4="GRANT ALL PRIVILEGES ON *.* TO daccord@'%' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q5="GRANT ALL PRIVILEGES ON *.* TO daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q6="GRANT ALL PRIVILEGES ON *.* TO root@'%' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q7="GRANT ALL PRIVILEGES ON *.* TO root@'localhost' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q8="FLUSH PRIVILEGES;"
                SQL="${Q1}${Q2}${Q3}${Q4}${Q5}${Q6}${Q7}${Q8}"
                ${MYSQL_USER} -uroot -p${MYSQL_ROOT_PASSWORD} -e "${SQL}" &

                systemctl enable mariadb.service >> ./${LOGFILE} 2>&1

                echo -e "\nThe mariadb server has been successfully prepared. Please note that a user (daccord) with the password (daccord) has been created."
            else
                echo "[${NOW} install.sh] An error occured while trying to install the mariadb server through yum." >> ./${LOGFILE}
                echo -e "\nAn error occured while trying to install the mariadb server through zypper, please view the ${LOGFILE} file in the directory where the script file is located."
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nYou have chosen not to install a local mariadb server."
            echo -e "\nPlease have your login credentials on hand for the configuration at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

# Give the user the option to install neo4j on a SUSE system through zypper (online modus) if not already installed
CHECK_NEO4J_SUSE(){
    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        echo -e "\nThe System Database is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] neo4j is already installed on this system." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\n\033[0;36mThe daccord software requires a database to store data.\033[m"
            read -p $'\033[0;36mDo you want to install the System Database locally [l] or use an external [e] instance? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install the System Database locally or [e] to use an external instance!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install neo4j locally." >> ./${LOGFILE}
            echo ""
            echo -n "Please wait while the System Database is being installed..."
            echo "[${NOW} install.sh] Checking if the required 'insserv-compat' package is already installed." >> ./${LOGFILE}
            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The 'insserv-compat' package is already installed, proceeding with neo4j installation." >> ./${LOGFILE}
            else
                echo "[${NOW} install.sh] The 'insserv-compat' package is not yet installed, proceeding with 'insserv-compat' installation." >> ./${LOGFILE}
                zypper --non-interactive in insserv-compat >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    while true; do
                        rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            sleep 3
                            break
                        fi
                        echo -n '.'
                        sleep 3
                    done
                else
                    rpm -Uvh insserv-compat* >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        while true; do
                            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                            if [[ $? == 0 ]]; then
                                sleep 3
                                break
                            fi
                            echo -n '.'
                            sleep 3
                        done
                    else
                        echo "[${NOW} install.sh] Unable to install the 'insserv-compat' package" >> ./${LOGFILE}
                    fi
                fi
            fi
            zypper ar --refresh http://yum.neo4j.com/stable neo4j >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh neo4j >> ./${LOGFILE} 2>&1
            zypper --non-interactive in neo4j-4.4.3-1 >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                while true; do
                    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        sleep 3
                        break
                    fi
                    echo -n '.'
                    sleep 3
                done
            else
                rpm -Uvh neo4j* >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    while true; do
                        rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            sleep 3
                            break
                        fi
                        echo -n '.'
                        sleep 3
                    done
                else
                    echo "[${NOW} install.sh] An error occured while trying to install neo4j." >> ./${LOGFILE}
                    echo -n "an error occured while trying to install the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                    CANCEL
                fi
            fi
            zypper rr neo4j >> ./${LOGFILE} 2>&1
            if [[ $? != 0 ]]; then
                while true; do
                    zypper lr neo4j >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        zypper rr neo4j >> ./${LOGFILE} 2>&1
                    else
                        break
                    fi
                    sleep 3
                done
            fi

            wget -nv https://github.com/neo4j-contrib/neo4j-apoc-procedures/releases/download/4.4.0.1/apoc-4.4.0.1-all.jar -P /var/lib/neo4j/plugins/ >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The apoc-4.4.0.1-all.jar file has been successfully downloaded." >> ./${LOGFILE}
            else
                echo "[${NOW} install.sh] An error occured while trying to download the apoc-4.4.0.1-all.jar file." >> ./${LOGFILE}
            fi

            CHECK_JAVA_VERSION=$(java -version 2>&1 | grep -o --max-count=1 '"11.')
            if [[ ${CHECK_JAVA_VERSION} != '"11.' ]]; then
                update-alternatives --set java /usr/lib64/jvm/jre-11-openjdk/bin/java >> ./${LOGFILE} 2>&1
            fi
            sed -i "s/#dbms.default_listen_address/dbms.default_listen_address/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.default_database=neo4j/dbms.default_database=daccord.db/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/dbms.connector.https.enabled=true/dbms.connector.https.enabled=false/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.security.procedures.unrestricted=my.extensions.example,my.procedures.*/dbms.security.procedures.unrestricted=algo.*,apoc.*/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1

            NEO4J_PASSWORD="daccord"
            neo4j-admin set-initial-password ${NEO4J_PASSWORD} >> ./${LOGFILE} 2>&1

            systemctl enable neo4j.service >> ./${LOGFILE} 2>&1
            systemctl start neo4j.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] neo4j has been successfully installed." >> ./${LOGFILE}
                echo -ne "System Database installed successfully\n"
                echo -n "Please wait while the System Database is being configured. This may take several minutes..."
                neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                while [[ ${neo_check} = "" ]]; do
                    echo -n '.'
                    sleep 3
                    neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                done
                sleep 10

                cypher-shell --username neo4j --password ${NEO4J_PASSWORD} --database system 'CREATE USER daccord SET PASSWORD "daccord" CHANGE NOT REQUIRED;' >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -ne "System Database configured successfully\n"
                    echo -e "\nPlease note that a user 'daccord' with the associated password 'daccord' has been created."
                else
                    echo "[${NOW} install.sh] An error occured while trying to prepare neo4j." >> ./${LOGFILE}
                    echo -n "an error occured while trying to prepare the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                    CANCEL
                fi
            else
                echo "[${NOW} install.sh] An error occured while trying to start neo4j." >> ./${LOGFILE}
                echo -n "an error occured while trying to start the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install neo4j locally." >> ./${LOGFILE}
            echo -e "\nPlease have your login credentials on hand for the configuration at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

# Give the user the option to install neo4j on a RedHat system through yum (online modus) if not already installed
CHECK_NEO4J_REDHAT(){
    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        echo -e "\nThe System Database is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] neo4j is already installed on this system." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\n\033[0;36mThe daccord software requires a database to store data.\033[m"
            read -p $'\033[0;36mDo you want to install the System Database locally [l] or use an external [e] instance? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install the System Database locally or [e] to use an external instance!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install neo4j locally." >> ./${LOGFILE}
            echo ""
            echo -n "Please wait while the System Database is being installed..."
            echo "[${NOW} install.sh] Checking if the required 'insserv-compat' package is already installed." >> ./${LOGFILE}
            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The 'insserv-compat' package is already installed, proceeding with neo4j installation." >> ./${LOGFILE}
            else
                echo "[${NOW} install.sh] The 'insserv-compat' package is not yet installed, proceeding with 'insserv-compat' installation." >> ./${LOGFILE}
                yum install -y insserv-compat >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    while true; do
                        rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            sleep 3
                            break
                        fi
                        echo -n '.'
                        sleep 3
                    done
                else
                    rpm -Uvh insserv-compat* >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        while true; do
                            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                            if [[ $? == 0 ]]; then
                                sleep 3
                                break
                            fi
                            echo -n '.'
                            sleep 3
                        done
                    else
                        echo "[${NOW} install.sh] Unable to install the 'insserv-compat' package" >> ./${LOGFILE}
                    fi
                fi
            fi
            rpm --import https://debian.neo4j.com/neotechnology.gpg.key
cat <<EOF>  /etc/yum.repos.d/neo4j.repo
[neo4j]
name=Neo4j RPM Repository
baseurl=https://yum.neo4j.com/stable
enabled=1
gpgcheck=1
EOF
            yum install -y neo4j-4.4.3-1 >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                while true; do
                    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        sleep 3
                        break
                    fi
                    echo -n '.'
                    sleep 3
                done
            else
                rpm -Uvh neo4j* >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    while true; do
                        rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            sleep 3
                            break
                        fi
                        echo -n '.'
                        sleep 3
                    done
                else
                    echo "[${NOW} install.sh] An error occured while trying to install neo4j." >> ./${LOGFILE}
                    echo -n "an error occured while trying to install the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                    CANCEL
                fi
            fi
            rm -fR /etc/yum.repos.d/neo4j.repo >> ./${LOGFILE} 2>&1

            wget -nv https://github.com/neo4j-contrib/neo4j-apoc-procedures/releases/download/4.4.0.1/apoc-4.4.0.1-all.jar -P /var/lib/neo4j/plugins/ >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The apoc-4.4.0.1-all.jar file has been successfully downloaded." >> ./${LOGFILE}
            else
                echo "[${NOW} install.sh] An error occured while trying to download the apoc-4.4.0.1-all.jar file." >> ./${LOGFILE}
            fi

            CHECK_JAVA_VERSION=$(java -version 2>&1 | grep -o --max-count=1 '"11.')
            if [[ ${CHECK_JAVA_VERSION} != '"11.' ]]; then
                update-alternatives --set java /usr/lib64/jvm/jre-11-openjdk/bin/java >> ./${LOGFILE} 2>&1
            fi
            sed -i "s/#dbms.default_listen_address/dbms.default_listen_address/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.default_database=neo4j/dbms.default_database=daccord.db/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/dbms.connector.https.enabled=true/dbms.connector.https.enabled=false/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.security.procedures.unrestricted=my.extensions.example,my.procedures.*/dbms.security.procedures.unrestricted=algo.*,apoc.*/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1

            NEO4J_PASSWORD="daccord"
            neo4j-admin set-initial-password ${NEO4J_PASSWORD} >> ./${LOGFILE} 2>&1

            systemctl enable neo4j.service >> ./${LOGFILE} 2>&1
            systemctl start neo4j.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] neo4j has been successfully installed." >> ./${LOGFILE}
                echo -ne "System Database installed successfully\n"
                echo -n "Please wait while the System Database is being configured. This may take several minutes..."
                neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                while [[ ${neo_check} = "" ]]; do
                    echo -n '.'
                    sleep 3
                    neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                done
                sleep 10

                cypher-shell --username neo4j --password ${NEO4J_PASSWORD} --database system 'CREATE USER daccord SET PASSWORD "daccord" CHANGE NOT REQUIRED;' >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -ne "System Database configured successfully\n"
                    echo -e "\nPlease note that a user 'daccord' with the associated password 'daccord' has been created."
                else
                    echo "[${NOW} install.sh] An error occured while trying to prepare neo4j." >> ./${LOGFILE}
                    echo -n "an error occured while trying to prepare the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                    CANCEL
                fi
            else
                echo "[${NOW} install.sh] An error occured while trying to start neo4j." >> ./${LOGFILE}
                echo -n "an error occured while trying to start the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install neo4j locally." >> ./${LOGFILE}
            echo -e "\nPlease have your login credentials on hand for the configuration at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

# Give the user the option to install through rpm (offline modus) if not already installed
CHECK_NEO4J_OFFLINE(){
    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        echo -e "\nThe System Database is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] neo4j is already installed on this system." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\n\033[0;36mThe daccord software requires a database to store data.\033[m"
            read -p $'\033[0;36mDo you want to install the System Database locally [l] or use an external [e] instance? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install the System Database locally or [e] to use an external instance!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install neo4j locally." >> ./${LOGFILE}
            echo ""
            echo -n "Please wait while the System Database is being installed..."
            echo "[${NOW} install.sh] Checking if the required 'insserv-compat' package is already installed." >> ./${LOGFILE}
            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The 'insserv-compat' package is already installed, proceeding with neo4j installation." >> ./${LOGFILE}
            else
                echo "[${NOW} install.sh] The 'insserv-compat' package is not yet installed, proceeding with 'insserv-compat' installation." >> ./${LOGFILE}
                zypper --non-interactive in insserv-compat >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    while true; do
                        rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            sleep 3
                            break
                        fi
                        echo -n '.'
                        sleep 3
                    done
                else
                    rpm -Uvh insserv-compat* >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        while true; do
                            rpm -qa | grep -i '^insserv-compat-[0-9]' >> /dev/null 2>&1
                            if [[ $? == 0 ]]; then
                                sleep 3
                                break
                            fi
                            echo -n '.'
                            sleep 3
                        done
                    else
                        echo "[${NOW} install.sh] Unable to install the 'insserv-compat' package" >> ./${LOGFILE}
                    fi
                fi
            fi
            rpm -Uvh --nodeps cypher-shell-4.4.3-1.noarch.rpm neo4j-4.4.3-1.noarch.rpm >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                while true; do
                    rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        sleep 3
                        break
                    fi
                    echo -n '.'
                    sleep 3
                done
            else
                echo "[${NOW} install.sh] An error occured while trying to install neo4j." >> ./${LOGFILE}
                echo -n "an error occured while trying to install the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                CANCEL
            fi

            mv ./apoc-4.4.0.1-all.jar /var/lib/neo4j/plugins

            CHECK_JAVA_VERSION=$(java -version 2>&1 | grep -o --max-count=1 '"11.')
            if [[ ${CHECK_JAVA_VERSION} != '"11.' ]]; then
                update-alternatives --set java /usr/lib64/jvm/jre-11-openjdk/bin/java >> ./${LOGFILE} 2>&1
            fi
            sed -i "s/#dbms.default_listen_address/dbms.default_listen_address/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.default_database=neo4j/dbms.default_database=daccord.db/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/dbms.connector.https.enabled=true/dbms.connector.https.enabled=false/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            sed -i "s/#dbms.security.procedures.unrestricted=my.extensions.example,my.procedures.*/dbms.security.procedures.unrestricted=algo.*,apoc.*/g" /etc/neo4j/neo4j.conf >> ./${LOGFILE} 2>&1
            echo 'dbms.jvm.additional=-Djava.net.preferIPv4Stack=true' >> /etc/neo4j/neo4j.conf

            NEO4J_PASSWORD="daccord"
            neo4j-admin set-initial-password ${NEO4J_PASSWORD} >> ./${LOGFILE} 2>&1

            systemctl enable neo4j.service >> ./${LOGFILE} 2>&1
            systemctl start neo4j.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] neo4j has been successfully installed." >> ./${LOGFILE}
                echo -ne "System Database installed successfully\n"
                echo -n "Please wait while the System Database is being configured. This may take several minutes..."
                neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                while [[ ${neo_check} = "" ]]; do
                    echo -n '.'
                    sleep 3
                    neo_check=$(systemctl status neo4j.service | grep "Remote interface available")
                done
                sleep 10

                cypher-shell --username neo4j --password ${NEO4J_PASSWORD} --database system 'CREATE USER daccord SET PASSWORD "daccord" CHANGE NOT REQUIRED;' >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -ne "System Database configured successfully\n"
                    echo -e "\nPlease note that a user 'daccord' with the associated password 'daccord' has been created."
                else
                    echo "[${NOW} install.sh] An error occured while trying to prepare neo4j." >> ./${LOGFILE}
                    echo -n "an error occured while trying to prepare the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                    CANCEL
                fi
            else
                echo "[${NOW} install.sh] An error occured while trying to start neo4j." >> ./${LOGFILE}
                echo -n "an error occured while trying to start the System Database, please refer to logfile: ${PWD}/${LOGFILE}"
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install neo4j locally." >> ./${LOGFILE}
            echo -e "\nPlease have your login credentials on hand for the configuration at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

# Install mariadb on a SUSE or Red Hat system through rpm (offline modus)
CHECK_MYSQL_LOCAL_SUSE(){
    if [[ ${REDHAT} == "TRUE" ]]; then
        rpm -qa | grep -i '^mariadb-server-[0-9]' >> /dev/null 2>&1
    else
        rpm -qa | grep -i '^mariadb-[0-9]' >> /dev/null 2>&1
    fi
    if [[ $? == 0 ]]; then
        echo -e "\nA mariadb server is already installed on this system. Please have your login credentials on hand for the configuration at a later stage."
        echo "[${NOW} install.sh] A mariadb server is already installed on the server." >> ./${LOGFILE}
    else
        EINGABE=
        while [[ ${EINGABE} != [lLeE] ]]; do
            echo -e "\nThe daccord Microsoft Edition requires a mariadb server to store data."
            read -p $'\033[0;36mDo you want to install a local [l] mariadb server or use an already existing external [e] mariadb server? [l/e] \033[m' EINGABE
            if [[ ${EINGABE} != [lLeE] ]]; then
                echo -e "\n\033[0;35mPlease choose between [l] to install a local mariadb server or [e] to choose an external mariadb server!\033[m"
            fi
        done
        if [[ ${EINGABE} == [lL] ]]; then
            echo "[${NOW} install.sh] User has chosen to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nPlease wait while the mariadb server is being installed."
            rpm -qa | grep -i '^libmariadb3-[0-9]' >> /dev/null 2>&1
            if [[ $? != 0 ]]; then
                rpm -Uvh libmariadb* >> /dev/null 2>&1
            fi
            rpm -qa | grep -i '^libopenssl1_1-[0-9]' >> /dev/null 2>&1
            if [[ $? != 0 ]]; then
                rpm -Uvh libopenssl* >> /dev/null 2>&1
            fi
            rpm -qa | grep -i '^unixODBC-[0-9]' >> /dev/null 2>&1
            if [[ $? != 0 ]]; then
                rpm -Uvh unixODBC* >> /dev/null 2>&1
            fi
            rpm -Uvh mariadb* >> ./${LOGFILE} 2>&1
            if [[ ${REDHAT} == "TRUE" ]]; then
                while true; do
                    rpm -qa | grep -i '^mariadb-server-[0-9]' >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        sleep 15
                        break
                    fi
                    grep "Failed dependencies" >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        break
                    fi
                    echo -n '.'
                    sleep 1
                done
            else
                while true; do
                    rpm -qa | grep -i '^mariadb-[0-9]' >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        sleep 2
                        break
                    fi
                    grep "Failed dependencies" >> /dev/null 2>&1
                    if [[ $? == 0 ]]; then
                        break
                    fi
                    echo -n '.'
                    sleep 1
                done
            fi
            systemctl start mariadb.service >> ./${LOGFILE} 2>&1
            if [[ $? == 0 ]]; then
                echo "[${NOW} install.sh] The local mariadb server has been successfully installed." >> ./${LOGFILE}
                echo -e "\nThe local mariadb server has been successfully installed."
                echo -e "\nPlease wait while the mariadb server is being prepared. This may take several minutes."
                rpm -qa | grep -i '^expect-[0-9]' >> /dev/null 2>&1
                if [[ $? != 0 ]]; then
                    rpm -Uvh expect* >> ./${LOGFILE} 2>&1
                fi

                MYSQLPW=
                MYSQL_ROOT_PASSWORD=daccord
                SECURE_MYSQL=$(expect -c "

                set timeout 10
                spawn mysql_secure_installation

                expect \"Enter current password for root (enter for none):\"
                send \"${MYSQLPW}\r\"

                expect \"Change the root password?\"
                send \"y\r\"

                expect \"New password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Re-enter new password:\"
                send \"${MYSQL_ROOT_PASSWORD}\r\"

                expect \"Remove anonymous users?\"
                send \"y\r\"

                expect \"Disallow root login remotely?\"
                send \"y\r\"

                expect \"Remove test database and access to it?\"
                send \"y\r\"

                expect \"Reload privilege tables now?\"
                send \"y\r\"

                expect eof
                ")

                mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                while [[ ${mariadb_check} = "" ]]; do
                    echo -n '.'
                    sleep 1
                    mariadb_check=$(systemctl status mariadb.service | grep -ie "Started M")
                done

                echo "${SECURE_MYSQL}" >> /dev/null &

                MYSQL_USER=`which mysql`

                sed -i s/bind-address/#bind-address/g /etc/my.cnf >> /dev/null 2>&1

                Q1="CREATE USER daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C';"
                Q2="CREATE DATABASE daccord;"
                Q3="CREATE DATABASE inchorus;"
                Q4="GRANT ALL PRIVILEGES ON *.* TO daccord@'%' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q5="GRANT ALL PRIVILEGES ON *.* TO daccord@'localhost' IDENTIFIED BY PASSWORD '*DA99DB18FDD33D5BD0D2C674B547D343DC6A502C' WITH GRANT OPTION;"
                Q6="GRANT ALL PRIVILEGES ON *.* TO root@'%' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q7="GRANT ALL PRIVILEGES ON *.* TO root@'localhost' IDENTIFIED BY '$MYSQL_ROOT_PASSWORD' WITH GRANT OPTION;"
                Q8="FLUSH PRIVILEGES;"
                SQL="${Q1}${Q2}${Q3}${Q4}${Q5}${Q6}${Q7}${Q8}"
                ${MYSQL_USER} -uroot -p${MYSQL_ROOT_PASSWORD} -e "${SQL}" &

                systemctl enable mariadb.service >> ./${LOGFILE} 2>&1

                echo -e "\nThe mariadb server has been successfully prepared. Please note that a user (daccord) with the password (daccord) has been created."
            else
                echo "[${NOW} install.sh] An error occured while trying to install the mariadb server through rpm." >> ./${LOGFILE}
                echo -e "\n\033[0;35mAn error occured while trying to install the mariadb server through rpm, please view the ${LOGFILE} file in the directory where the script file is located.\033[m"
                CANCEL
            fi
        else
            echo "[${NOW} install.sh] User has chosen not to install a local mariadb server." >> ./${LOGFILE}
            echo -e "\nYou have chosen not to install a local mariadb server."
            echo -e "\nPlease have your login credentials on hand for the web setup at a later stage."
            PRESS_ANY_KEY
        fi
    fi
}

ONLINE_INSTALL(){
    echo "[${NOW} install.sh] User has requested to start the online installation." >> ./${LOGFILE}
    echo -e "\n"
    EINGABE=
    while [[ ${EINGABE} != [iI] ]]; do
        read -p $'\033[0;36mThe required third party components have been prepared. Please enter [i] to install the daccord Microsoft Edition software now. [i] \033[m' EINGABE
        if [[ ${EINGABE} != [iI] ]]; then
            echo -e "\n\033[0;35mPlease enter [i] to install the daccord Microsoft Edition software now. [i] \033[m"
        fi
    done
    if [[ ${EINGABE} == [iI] ]]; then
        echo -e "\nPlease wait while the daccord Microsoft Edition software is being installed."
        INSTERROR=0
        if [[ ${REDHAT} == "TRUE" ]]; then
            ONLINE_REPO_INSTALL_REDHAT
        else
            ONLINE_REPO_INSTALL_SUSE
        fi
        if [[ $INSTERROR == "0" ]]; then
            echo -e "\nThe daccord Microsoft Edition software components have been installed successfully!"
            echo -e "\n\033[0;36mTo finalize your installation, please access the URL http://[Server-IP]:55080/system/ with a web-browser to configure your daccord system.\033[m"
            echo -e "\n"
            rm -f /opt/GuH/daccord_mse_install_flag >> /dev/null
        else
            echo -e "\n"
            echo -e "\033[1;31mSomething went wrong! An error was detected during the installation of the daccord Microsoft Edition software components!\033[m"
            echo -e "\033[1;31mTo troubleshoot the installation please view the ${LOGFILE} file in the directory where the script file is located.\033[m"
            echo -e "\n"
        fi
    else
        CANCEL
    fi
}

ONLINE_REPO_INSTALL_SUSE(){
    echo "[${NOW} install.sh] Installing the daccord Microsoft Edition components from the repository." >> ./${LOGFILE}
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        zypper lr daccord-msedition-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1-dev daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-persondata-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1-dev daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-msfileserver-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1-dev daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-activedirectory-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1-dev daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        fi
    else
        zypper lr daccord-msedition-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1 daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-persondata-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1 daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-msfileserver-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1 daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-activedirectory-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1 daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        fi
    fi
    zypper --non-interactive in RuntimeEngine >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl enable RuntimeEngine.service >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl start RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine installed successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while installing the RuntimeEngine"
        echo "[${NOW} install.sh] Error while installing the RuntimeEngine." >> ./${LOGFILE}
    fi
    zypper --non-interactive in inchorus-portal-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
        echo -n '.'
        echo "[${NOW} install.sh] inchorus-portal-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the inchorus-portal-gadget"
        echo "[${NOW} install.sh] Error while installing the inchorus-portal-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-system-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-system-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-collector-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-collector-engine." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-policy-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-policy-engine." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while installing the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-csv-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while installing the daccord-csv-connector." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-portal-customfiles >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while installing the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-myperson-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-persons-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive in daccord-policies-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policies-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
    fi
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        zypper mr -d daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
    else
        zypper mr -d daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
    fi
}

ONLINE_REPO_INSTALL_REDHAT(){
    echo "[${NOW} install.sh] Installing the daccord Microsoft Edition components from the repository." >> ./${LOGFILE}
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        if [[ -f "/etc/yum.repos.d/daccord-msedition-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-msedition-2.1-dev.repo
[daccord-msedition-2.1-dev]
name=daccord msedition dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-persondata-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-persondata-2.1-dev.repo
[daccord-ip-persondata-2.1-dev]
name=daccord ip persondata dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-msfileserver-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-msfileserver-2.1-dev.repo
[daccord-ip-msfileserver-2.1-dev]
name=daccord ip msfileserver dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-activedirectory-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-activedirectory-2.1-dev.repo
[daccord-ip-activedirectory-2.1-dev]
name=daccord ip activedirectory dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
    else
        if [[ -f "/etc/yum.repos.d/daccord-msedition-2.1.repo" ]]; then
            yum config-manager --enable daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-msedition-2.1.repo
[daccord-msedition-2.1]
name=daccord msedition dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-persondata-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-persondata-2.1.repo
[daccord-ip-persondata-2.1]
name=daccord ip persondata dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-msfileserver-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-msfileserver-2.1.repo
[daccord-ip-msfileserver-2.1]
name=daccord ip msfileserver dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-activedirectory-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-activedirectory-2.1.repo
[daccord-ip-activedirectory-2.1]
name=daccord ip activedirectory dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1
enabled=1
gpgcheck=0
EOF
        fi
    fi
    echo -n '.'
    yum install -y RuntimeEngine >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl enable RuntimeEngine.service >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl start RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine installed successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while installing the RuntimeEngine"
        echo "[${NOW} install.sh] Error while installing the RuntimeEngine." >> ./${LOGFILE}
    fi
    yum install -y inchorus-portal-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
        echo -n '.'
        echo "[${NOW} install.sh] inchorus-portal-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the inchorus-portal-gadget"
        echo "[${NOW} install.sh] Error while installing the inchorus-portal-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-system-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-system-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-collector-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-collector-engine." >> ./${LOGFILE}
    fi
    yum install -y daccord-policy-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-policy-engine." >> ./${LOGFILE}
    fi
    yum install -y daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while installing the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    yum install -y daccord-csv-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while installing the daccord-csv-connector." >> ./${LOGFILE}
    fi
    yum install -y daccord-portal-customfiles >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while installing the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    yum install -y daccord-myperson-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-persons-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    yum install -y daccord-policies-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policies-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
    fi
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        yum config-manager --disable daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
    else
        yum config-manager --disable daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
    fi
}

LOCAL_INSTALL(){
    echo "[${NOW} install.sh] User has requested to start the local installation." >> ./${LOGFILE}
    echo -e "\n"
    EINGABE=
    while [[ ${EINGABE} != [iI] ]]; do
        read -p $'\033[0;36mThe required third party components have been prepared. Please enter [i] to install the daccord Microsoft Edition software now. [i] \033[m' EINGABE
        if [[ ${EINGABE} != [iI] ]]; then
            echo -e "\n\033[0;35mPlease enter [i] to install the daccord Microsoft Edition software now. [i]\033[m"
        fi
    done
    if [[ ${EINGABE} == [iI] ]]; then
        echo -e "\nPlease wait while the daccord Microsoft Edition software is being installed."
        INSTERROR=0
        LOCAL_RPM_INSTALL
        if [[ $INSTERROR == "0" ]]; then
            echo -e "[${NOW} install.sh] The daccord Microsoft Edition software components have been installed successfully!" >> ./${LOGFILE}
            echo -e "\nThe daccord Microsoft Edition software components have been installed successfully!"
            echo -e "\n\033[0;36mTo finalize your installation, please access the URL http://[Server-IP]:55080/system/ with a web-browser to configure your daccord system.\033[m"
            echo -e "\n"
            rm -f /opt/GuH/daccord_mse_install_flag
        else
            echo -e "[${NOW} install.sh] Something went wrong! An error was detected during the installation of the daccord Microsoft Edition components!" >> ./${LOGFILE}
            echo -e "\033[1;31mSomething went wrong! An error was detected during the installation of the daccord Microsoft Edition components!\033[m"
            echo -e "\033[0;35mTo troubleshoot the installation please view the ${LOGFILE} file in the directory where the script file is located.\033[m"
            echo -e "\n"
        fi
    else
        CANCEL
    fi
}

LOCAL_RPM_INSTALL(){
    echo "[${NOW} install.sh] Installing the daccord Microsoft Edition components from local RPM files." >> ./${LOGFILE}
    rpm -Uvh RuntimeEngine* >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl enable RuntimeEngine.service >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl start RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine installed successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while installing the RuntimeEngine"
        echo "[${NOW} install.sh] Error while installing the RuntimeEngine." >> ./${LOGFILE}
    fi
    rpm -Uvh inchorus-portal-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
        echo -n '.'
        echo "[${NOW} install.sh] inchorus-portal-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the inchorus-portal-gadget"
        echo "[${NOW} install.sh] Error while installing the inchorus-portal-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-system-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-system-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-collector-engine* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-collector-engine." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-policy-engine* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while installing the daccord-policy-engine." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-activedirectory-connector* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while installing the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-csv-connector* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while installing the daccord-csv-connector." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-portal-customfiles* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while installing the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-myperson-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-persons-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-msfileserver-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-activedirectory-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-policies-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while installing the daccord-policies-gadget"
        echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
    fi
}

ONLINE_UPDATE(){
    echo "[${NOW} install.sh] User has requested to start the online update." >> ./${LOGFILE}
    INSTERROR=0
    if [[ ${REDHAT} == "TRUE" ]]; then
        ONLINE_REPO_UPDATE_REDHAT
    else
        ONLINE_REPO_UPDATE_SUSE
    fi
    if [[ $INSTERROR == "0" ]]; then
        echo -e "\nThe daccord Microsoft Edition software components have been updated successfully!"
        echo -e "\n\033[0;36mIMPORTANT! To finalize the update please access the URL http://[Server-IP]:55080/system/ with a web-browser!\033[m"
        echo -e "\n"
        EXIT
    else
        echo -e "\n"
        echo -e "\033[1;31mSomething went wrong! An error was detected during the update of the daccord Microsoft Edition software components!\033[m"
        echo -e "\033[1;31mTo troubleshoot the update please view the ${LOGFILE} file in the directory where the script file is located.\033[m"
        echo -e "\n"
        EXIT
    fi
}

ONLINE_REPO_UPDATE_SUSE(){
    echo "[${NOW} install.sh] User has requested to update the daccord Microsoft Edition components." >> ./${LOGFILE}
    echo -e "\nPlease wait while the daccord Microsoft Edition software components are being updated."
    zypper lr daccord-msedition-2.0 >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper rr daccord-msedition-2.0 >> ./${LOGFILE} 2>&1
    fi
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        zypper lr daccord-msedition-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1-dev daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-persondata-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1-dev daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-msfileserver-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1-dev daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-activedirectory-2.1-dev >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1-dev daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        fi
    else
        zypper lr daccord-msedition-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1 daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-persondata-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1 daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-msfileserver-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1 daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        fi
        zypper lr daccord-ip-activedirectory-2.1 >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            zypper mr -e daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
            zypper refresh daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        else
            zypper ar http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1 daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
            zypper --gpg-auto-import-keys refresh daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        fi
    fi
    echo -n '.'
    systemctl stop RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    sleep 10
    echo -n '.'
    rpm -qa | grep daccord-activedirectory-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive rm daccord-activedirectory-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-activedirectory-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-activedirectory-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-activedirectory-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-msfileserver-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive rm daccord-msfileserver-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msfileserver-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msfileserver-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-msfileserver-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-person-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive rm daccord-person-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-person-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-person-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-person-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-system-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive rm daccord-system-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-system-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-system-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-system-policies." >> ./${LOGFILE}
        fi
    fi
    if [[ -e /opt/GuH/inchorus-portal ]]; then
        zypper --non-interactive rm inchorus-portal >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            rm -fR /opt/GuH/inchorus-portal /var/opt/GuH/inchorus-portal >> ./${LOGFILE} 2>&1
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the inchorus-portal"
            echo "[${NOW} install.sh] Error while removing the inchorus-portal." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep inchorus-portal-gadget >> /dev/null 2>&1
    if  [[ $? == 0 ]]; then
        zypper --non-interactive up inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the inchorus-portal-gadget"
            echo "[${NOW} install.sh] Error while updating the inchorus-portal-gadget." >> ./${LOGFILE}
        fi
    else
        zypper --non-interactive in inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal-gadget installed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while installing the inchorus-portal-gadget"
            echo "[${NOW} install.sh] Error while installing the inchorus-portal-gadget." >> ./${LOGFILE}
        fi
    fi
    zypper --non-interactive up daccord-system-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-system-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-collector-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-collector-engine." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-policy-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-policy-engine." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while updating the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-csv-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while updating the daccord-csv-connector." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-portal-customfiles >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while updating the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-msedition-doc >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive rm daccord-msedition-doc >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msedition-doc removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msedition-doc"
            echo "[${NOW} install.sh] Error while removing the daccord-msedition-doc." >> ./${LOGFILE}
        fi
    fi
    zypper --non-interactive up daccord-myperson-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-persons-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    zypper --non-interactive up daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-policies-gadget >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper --non-interactive up daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while updating the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    else
        zypper --non-interactive in daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while installing the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    fi
    zypper --non-interactive up RuntimeEngine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine updated successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while updating the RuntimeEngine"
        echo "[${NOW} install.sh] Error while updating the RuntimeEngine." >> ./${LOGFILE}
    fi
    systemctl start RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        zypper mr -d daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
    else
        zypper mr -d daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        zypper mr -d daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
    fi
}

ONLINE_REPO_UPDATE_REDHAT(){
    echo "[${NOW} install.sh] User has requested to update the daccord Microsoft Edition components." >> ./${LOGFILE}
    echo -e "\nPlease wait while the daccord Microsoft Edition software components are being updated."
    if [[ -f "/etc/yum.repos.d/daccord-msedition-2.0.repo" ]]; then
        rm -fR /etc/yum.repos.d/daccord-msedition-2.0.repo >> ./${LOGFILE} 2>&1
    fi
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        if [[ -f "/etc/yum.repos.d/daccord-msedition-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-msedition-2.1-dev.repo
[daccord-msedition-2.1-dev]
name=daccord msedition dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-persondata-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-persondata-2.1-dev.repo
[daccord-ip-persondata-2.1-dev]
name=daccord ip persondata dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-msfileserver-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-msfileserver-2.1-dev.repo
[daccord-ip-msfileserver-2.1-dev]
name=daccord ip msfileserver dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-activedirectory-2.1-dev.repo" ]]; then
            yum config-manager --enable daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-activedirectory-2.1-dev.repo
[daccord-ip-activedirectory-2.1-dev]
name=daccord ip activedirectory dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1-dev
enabled=1
gpgcheck=0
EOF
        fi
    else
        if [[ -f "/etc/yum.repos.d/daccord-msedition-2.1.repo" ]]; then
            yum config-manager --enable daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-msedition-2.1.repo
[daccord-msedition-2.1]
name=daccord msedition dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-msedition-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-persondata-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-persondata-2.1.repo
[daccord-ip-persondata-2.1]
name=daccord ip persondata dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-persondata-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-msfileserver-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-msfileserver-2.1.repo
[daccord-ip-msfileserver-2.1]
name=daccord ip msfileserver dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-msfileserver-2.1
enabled=1
gpgcheck=0
EOF
        fi
        if [[ -f "/etc/yum.repos.d/daccord-ip-activedirectory-2.1.repo" ]]; then
            yum config-manager --enable daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        else
cat <<EOF>  /etc/yum.repos.d/daccord-ip-activedirectory-2.1.repo
[daccord-ip-activedirectory-2.1]
name=daccord ip activedirectory dev RPM Repository
baseurl=http://mirror.guh-systems.de/products/GuH/repos/update/daccord-ip-activedirectory-2.1
enabled=1
gpgcheck=0
EOF
        fi
    fi
    yum clean all >> /dev/null 2>&1
    echo -n '.'
    systemctl stop RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    sleep 10
    echo -n '.'
    rpm -qa | grep daccord-activedirectory-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum remove -y daccord-activedirectory-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-activedirectory-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-activedirectory-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-activedirectory-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-msfileserver-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum remove -y daccord-msfileserver-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msfileserver-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msfileserver-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-msfileserver-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-person-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum remove -y daccord-person-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-person-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-person-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-person-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-system-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum remove -y daccord-system-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-system-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-system-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-system-policies." >> ./${LOGFILE}
        fi
    fi
    if [[ -e /opt/GuH/inchorus-portal ]]; then
        yum remove -y inchorus-portal >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            rm -fR /opt/GuH/inchorus-portal /var/opt/GuH/inchorus-portal >> ./${LOGFILE} 2>&1
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the inchorus-portal"
            echo "[${NOW} install.sh] Error while removing the inchorus-portal." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep inchorus-portal-gadget >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum upgrade -y inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the inchorus-portal-gadget"
            echo "[${NOW} install.sh] Error while updating the inchorus-portal-gadget." >> ./${LOGFILE}
        fi
    else
        yum install -y inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the inchorus-portal-gadget"
            echo "[${NOW} install.sh] Error while updating the inchorus-portal-gadget." >> ./${LOGFILE}
        fi
    fi
    yum upgrade -y daccord-system-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-system-gadget." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-collector-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-collector-engine." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-policy-engine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-policy-engine." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while updating the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-csv-connector >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while updating the daccord-csv-connector." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-portal-customfiles >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while updating the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-msedition-doc >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum remove -y daccord-msedition-doc >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msedition-doc removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msedition-doc"
            echo "[${NOW} install.sh] Error while removing the daccord-msedition-doc." >> ./${LOGFILE}
        fi
    fi
    yum upgrade -y daccord-myperson-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-persons-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    yum upgrade -y daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-policies-gadget >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        yum upgrade -y daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while updating the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    else
        yum install -y daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while installing the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    fi
    yum upgrade -y RuntimeEngine >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine updated successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while updating the RuntimeEngine"
        echo "[${NOW} install.sh] Error while updating the RuntimeEngine." >> ./${LOGFILE}
    fi
    systemctl start RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
    if [[ ${STAGE} == "DEVELOPMENT" ]]; then
        yum config-manager --disable daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
    else
        yum config-manager --disable daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        yum config-manager --disable daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
    fi
}

LOCAL_UPDATE(){
    echo "[${NOW} install.sh] User has requested to start the local update." >> ./${LOGFILE}
    echo -e "\nPlease wait while the daccord Microsoft Edition software is being updated."
    INSTERROR=0
    LOCAL_RPM_UPDATE
    if [[ $INSTERROR == "0" ]]; then
        echo -e "\nThe daccord Microsoft Edition software components have been updated successfully!"
        echo -e "\n"
    else
        echo -e "\n"
        echo -e "\033[1;31mSomething went wrong! An error was detected during the update of the daccord Microsoft Edition software components!\033[m"
        echo -e "\033[1;31mTo troubleshoot the update please view the ${LOGFILE} file in the directory where the script file is located.\033[m"
        echo -e "\n"
    fi
}

LOCAL_RPM_UPDATE(){
    echo "[${NOW} install.sh] User has requested to update the daccord Microsoft Edition components." >> ./${LOGFILE}
    echo -n '.'
    systemctl stop RuntimeEngine.service >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    sleep 10
    echo -n '.'
    rpm -qa | grep daccord-activedirectory-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-activedirectory-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-activedirectory-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-activedirectory-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-activedirectory-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-msfileserver-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-msfileserver-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msfileserver-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msfileserver-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-msfileserver-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-person-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-person-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-person-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-person-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-person-policies." >> ./${LOGFILE}
        fi
    fi
    rpm -qa | grep daccord-system-policies >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-system-policies >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-system-policies removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-system-policies"
            echo "[${NOW} install.sh] Error while removing the daccord-system-policies." >> ./${LOGFILE}
        fi
    fi
    if [[ -e /opt/GuH/inchorus-portal ]]; then
        rpm -ve inchorus-portal >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            rm -rf /opt/GuH/inchorus-portal /var/opt/GuH/inchorus-portal >> /dev/null 2>&1
            echo -n '.'
            echo "[${NOW} install.sh] inchorus-portal removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the inchorus-portal"
            echo "[${NOW} install.sh] Error while removing the inchorus-portal." >> ./${LOGFILE}
        fi
    fi
    rpm -Uvh inchorus-portal-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] inchorus-portal-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the inchorus-portal-gadget"
        echo "[${NOW} install.sh] Error while updating the inchorus-portal-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-system-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-system-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-system-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-system-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-collector-engine* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-collector-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-collector-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-collector-engine." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-policy-engine* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-policy-engine updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-policy-engine"
        echo "[${NOW} install.sh] Error while updating the daccord-policy-engine." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-activedirectory-connector* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] dacoord-activedirectory-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-connector"
        echo "[${NOW} install.sh] Error while updating the ddaccord-activedirectory-connector." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-csv-connector* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-csv-connector updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-csv-connector"
        echo "[${NOW} install.sh] Error while updating the daccord-csv-connector." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-portal-customfiles* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-portal-customfiles updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-portal-customfiles"
        echo "[${NOW} install.sh] Error while updating the daccord-portal-customfiles." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-msedition-doc >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-msedition-doc >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-msedition-doc removed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while removing the daccord-msedition-doc"
            echo "[${NOW} install.sh] Error while removing the daccord-msedition-doc." >> ./${LOGFILE}
        fi
    fi
    rpm -Uvh daccord-myperson-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-myperson-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-myperson-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-myperson-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-persons-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-persons-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-persons-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-persons-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-msfileserver-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-msfileserver-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-msfileserver-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-msfileserver-gadget." >> ./${LOGFILE}
    fi
    rpm -Uvh daccord-activedirectory-gadget* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] daccord-activedirectory-gadget updated successfully." >> ./${LOGFILE}
    else
        INSTERROR=1
        echo -n "Error while updating the daccord-activedirectory-gadget"
        echo "[${NOW} install.sh] Error while updating the daccord-activedirectory-gadget." >> ./${LOGFILE}
    fi
    rpm -qa | grep daccord-policies-gadget >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -Uvh daccord-policies-gadget* >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget updated successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while updating the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while updating the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    else
        rpm -Uvh daccord-policies-gadget* >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
            echo "[${NOW} install.sh] daccord-policies-gadget installed successfully." >> ./${LOGFILE}
        else
            INSTERROR=1
            echo -n "Error while installing the daccord-policies-gadget"
            echo "[${NOW} install.sh] Error while installing the daccord-policies-gadget." >> ./${LOGFILE}
        fi
    fi
    rpm -Uvh RuntimeEngine* >> ./${LOGFILE} 2>&1
    if [[ $? == 0 ]]; then
        echo -n '.'
        echo "[${NOW} install.sh] RuntimeEngine updated successfully." >> ./${LOGFILE}
        sed -i 's/Before=multi-user.target graphical.target/Before=multi-user.target graphical.target\nAfter=neo4j.service mariadb.service/g' /usr/lib/systemd/system/RuntimeEngine.service >> ./${LOGFILE} 2>&1
    else
        INSTERROR=1
        echo -n "Error while updating the RuntimeEngine"
        echo "[${NOW} install.sh] Error while updating the RuntimeEngine." >> ./${LOGFILE}
    fi
    systemctl start RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    if [[ $? != 0 ]]; then
        INSTERROR=1
    fi
    echo "inchorus;daccord" > /opt/GuH/RuntimeEngine/bin/rename_war
    echo -e "\nThe daccord Microsoft Edition components have been updated."
    echo "[${NOW} install.sh] The daccord Microsoft Edition components have been updated." >> ./${LOGFILE}
    EXIT
}

UNINSTALL_SUSE(){
    DELERROR=0
    echo "[${NOW} install.sh] User has requested to remove the daccord Microsoft Edition components." >> ./${LOGFILE}
    echo -e "\nPlease wait while the daccord Microsoft Edition components are being removed."
    zypper lr daccord-msedition-2.1-dev >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper rr daccord-msedition-2.1-dev >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-persondata-2.1-dev >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-msfileserver-2.1-dev >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-activedirectory-2.1-dev >> ./${LOGFILE} 2>&1
        echo -n '.'
    fi
    zypper lr daccord-msedition-2.1 >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        zypper rr daccord-msedition-2.1 >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-persondata-2.1 >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-msfileserver-2.1 >> ./${LOGFILE} 2>&1
        echo -n '.'
        zypper rr daccord-ip-activedirectory-2.1 >> ./${LOGFILE} 2>&1
        echo -n '.'
    fi
    systemctl stop RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl disable RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    echo -n '.'
    rpm -qa | grep -i '^daccord-policies-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-policies-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-policies-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-activedirectory-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-activedirectory-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-activedirectory-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-msfileserver-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-msfileserver-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-msfileserver-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-persons-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-persons-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-persons-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-persons-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-myperson-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-myperson-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-myperson-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-myperson-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-portal-customfiles-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-portal-customfiles >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-portal-customfiles." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-portal-customfiles component."
        fi
    fi
    rpm -qa | grep -i '^daccord-csv-connector-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-csv-connector >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-csv-connector." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-csv-connector component."
        fi
    fi
    rpm -qa | grep -i '^daccord-activedirectory-connector-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-activedirectory-connector." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-activedirectory-connector component."
        fi
    fi
    rpm -qa | grep -i '^daccord-policy-engine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-policy-engine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-policy-engine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-policy-engine component."
        fi
    fi
    rpm -qa | grep -i '^daccord-collector-engine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-collector-engine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-collector-engine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-collector-engine component."
        fi
    fi
    rpm -qa | grep -i '^daccord-system-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-system-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-system-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-system-gadget component."
        fi
    fi
    rpm -qa | grep -i '^inchorus-portal-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling inchorus-portal-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the inchorus-portal-gadget component."
        fi
    fi
    rpm -qa | grep -i '^RuntimeEngine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve RuntimeEngine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling RuntimeEngine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the Runtime Engine component."
        fi
    fi
    if [[ ${DELERROR} == "0" ]]; then
        rm -fR /opt/GuH/ /var/opt/GuH/ >> ./${LOGFILE} 2>&1
        echo -n '.'
        echo -e "\nThe daccord Microsoft Edition components have been removed."
        echo "[${NOW} install.sh] The daccord Microsoft Edition components have been removed." >> ./${LOGFILE}
        rpm -qa | grep -i '^mariadb-[0-9]' >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            EINGABE=
            while [[ ${EINGABE} != [YyNn] ]]; do
                read -p $'\033[0;36mWould you like to completely remove the mariadb software as well? [y/n] \033[m' EINGABE
                if [[ ${EINGABE} != [YyNn] ]]; then
                    echo -e "\n\033[0;35mPlease choose between [y] to completely remove the mariadb software or [n] to skip!\033[m"
                fi
            done
            if [[ ${EINGABE} == [Yy] ]]; then
                echo "[${NOW} install.sh] User has requested to completely remove mariadb." >> ./${LOGFILE}
                echo -e "\nPlease wait while mariadb is being removed."
                echo -n '.'
                systemctl stop mariadb.service >> ./${LOGFILE} 2>&1
                echo -n '.'
                systemctl disable mariadb.service >> ./${LOGFILE} 2>&1
                echo -n '.'
                rpm -ve mariadb >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -n '.'
                else
                    DELERROR=1
                    echo "[${NOW} install.sh] Error while uninstalling mariadb." >> ./${LOGFILE}
                    echo -e "\nError while uninstalling the mariadb software."
                fi
                rpm -ve mariadb-client >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -n '.'
                else
                    DELERROR=1
                    echo "[${NOW} install.sh] Error while uninstalling mariadb-client." >> ./${LOGFILE}
                    echo -e "\nError while uninstalling the mariadb-client software."
                fi
                rpm -ve mariadb-errormessages >> ./${LOGFILE} 2>&1
                if [[ $? == 0 ]]; then
                    echo -n '.'
                else
                    DELERROR=1
                    echo "[${NOW} install.sh] Error while uninstalling mariadb-errormessages." >> ./${LOGFILE}
                    echo -e "\nError while uninstalling the mariadb-errormessages software."
                fi
                if [[ ${DELERROR} == "0" ]]; then
                    rm -fR /var/log/mysql/ /var/lib/mysql /run/mysql/ /usr/share/mariadb/ >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    echo -e "\nThe mariadb software has been successfully removed."
                    echo "[${NOW} install.sh] mariadb has been successfully removed." >> ./${LOGFILE}
                else
                    echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
                    echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
                fi
            fi
        fi
        rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            if [[ -d "/var/lib/neo4j/data/databases/daccord.db/" ]]; then
                EINGABE=
                while [[ ${EINGABE} != [YyNn] ]]; do
                    read -p $'\033[0;36mWould you like to completely remove the neo4j software as well? [y/n] \033[m' EINGABE
                    if [[ ${EINGABE} != [YyNn] ]]; then
                        echo -e "\n\033[0;35mPlease choose between [y] to completely remove the neo4j software or [n] to skip!\033[m"
                    fi
                done
                if [[ ${EINGABE} == [Yy] ]]; then
                    DELERROR=0
                    echo "[${NOW} install.sh] User has requested to remove neo4j." >> ./${LOGFILE}
                    echo -e "\nPlease wait while the neo4j software is being removed."
                    echo -n '.'
                    systemctl stop neo4j.service >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    systemctl disable neo4j.service >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    rpm -ve neo4j >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling neo4j." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the neo4j software."
                    fi
                    rpm -ve cypher-shell >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling cypher-shell." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling cypher-shell."
                    fi
                    if [[ ${DELERROR} == "0" ]]; then
                        rm -fR /var/log/neo4j/ /var/lib/neo4j/ /etc/neo4j/ /usr/share/doc/neo4j/ >> ./${LOGFILE} 2>&1
                        echo -n '.'
                        zypper lr neo4j >> /dev/null 2>&1
                        if [[ $? == 0 ]]; then
                            zypper rr neo4j >> ./${LOGFILE} 2>&1
                        fi
                        echo -e "\nThe neo4j software has been successfully removed."
                        echo "[${NOW} install.sh] neo4j has been successfully removed." >> ./${LOGFILE}
                    else
                        echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
                        echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
                    fi
                fi
            fi
        fi
    else
        echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
        echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
    fi
}

UNINSTALL_REDHAT(){
    DELERROR=0
    echo "[${NOW} install.sh] User has requested to remove the daccord Microsoft Edition components." >> ./${LOGFILE}
    echo -e "\nPlease wait while the daccord Microsoft Edition components are being removed."
    if [[ ! -f "/etc/yum.repos.d/daccord-msedition-2.1-dev.repo" ]]; then
        rm -f /etc/yum.repos.d/daccord-msedition-2.1-dev.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-persondata-2.1-dev.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-msfileserver-2.1-dev.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-activedirectory-2.1-dev.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
    fi
    if [[ ! -f "/etc/yum.repos.d/daccord-msedition-2.1.repo" ]]; then
        rm -f /etc/yum.repos.d/daccord-msedition-2.1.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-persondata-2.1.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-msfileserver-2.1.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
        rm -f /etc/yum.repos.d/daccord-ip-activedirectory-2.1.repo >> ./${LOGFILE} 2>&1
        echo -n '.'
    fi
    systemctl stop RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    echo -n '.'
    systemctl disable RuntimeEngine.service  >> ./${LOGFILE} 2>&1
    echo -n '.'
    rpm -qa | grep -i '^daccord-policies-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-policies-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-policies-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-policies-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-activedirectory-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-activedirectory-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-activedirectory-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-activedirectory-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-msfileserver-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-msfileserver-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-msfileserver-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-msfileserver-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-persons-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-persons-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-persons-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-persons-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-myperson-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-myperson-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-myperson-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-myperson-gadget component."
        fi
    fi
    rpm -qa | grep -i '^daccord-portal-customfiles-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-portal-customfiles >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-portal-customfiles." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-portal-customfiles component."
        fi
    fi
    rpm -qa | grep -i '^daccord-csv-connector-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-csv-connector >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-csv-connector." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-csv-connector component."
        fi
    fi
    rpm -qa | grep -i '^daccord-activedirectory-connector-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-activedirectory-connector >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-activedirectory-connector." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-activedirectory-connector component."
        fi
    fi
    rpm -qa | grep -i '^daccord-policy-engine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-policy-engine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-policy-engine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-policy-engine component."
        fi
    fi
    rpm -qa | grep -i '^daccord-collector-engine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-collector-engine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-collector-engine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-collector-engine component."
        fi
    fi
    rpm -qa | grep -i '^daccord-system-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve daccord-system-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling daccord-system-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the daccord-system-gadget component."
        fi
    fi
    rpm -qa | grep -i '^inchorus-portal-gadget-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve inchorus-portal-gadget >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling inchorus-portal-gadget." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the inchorus-portal-gadget component."
        fi
    fi
    rpm -qa | grep -i '^RuntimeEngine-[0-9]' >> /dev/null 2>&1
    if [[ $? == 0 ]]; then
        rpm -ve RuntimeEngine >> ./${LOGFILE} 2>&1
        if [[ $? == 0 ]]; then
            echo -n '.'
        else
            DELERROR=1
            echo "[${NOW} install.sh] Error while uninstalling RuntimeEngine." >> ./${LOGFILE}
            echo -e "\nError while uninstalling the Runtime Engine component."
        fi
    fi
    if [[ ${DELERROR} == "0" ]]; then
        rm -fR /opt/GuH/ /var/opt/GuH/ >> ./${LOGFILE} 2>&1
        echo -n '.'
        echo -e "\nThe daccord Microsoft Edition components have been removed."
        echo "[${NOW} install.sh] The daccord Microsoft Edition components have been removed." >> ./${LOGFILE}
        rpm -qa | grep -i '^mariadb-server-[0-9]' >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            EINGABE=
            while [[ ${EINGABE} != [YyNn] ]]; do
                read -p $'\033[0;36mWould you like to completely remove the mariadb software as well? [y/n] \033[m' EINGABE
                if [[ ${EINGABE} != [YyNn] ]]; then
                    echo -e "\n\033[0;35mPlease choose between [y] to completely remove the mariadb software or [n] to skip!\033[m"
                fi
            done
            if [[ ${EINGABE} == [Yy] ]]; then
                echo "[${NOW} install.sh] User has requested to completely remove mariadb." >> ./${LOGFILE}
                echo -e "\nPlease wait while mariadb is being removed."
                echo -n '.'
                systemctl stop mariadb.service >> ./${LOGFILE} 2>&1
                echo -n '.'
                systemctl disable mariadb.service >> ./${LOGFILE} 2>&1
                echo -n '.'
                rpm -qa | grep -i '^mariadb-backup-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve mariadb-backup >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling mariadb-backup." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the mariadb-backup software."
                    fi
                fi
                rpm -qa | grep -i '^mariadb-gssapi-server-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve mariadb-gssapi-server >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling mariadb-gssapi-server." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the mariadb-gssapi-server software."
                    fi
                fi
                rpm -qa | grep -i '^mariadb-server-utils-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve mariadb-server-utils >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling mariadb-server-utils." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the mariadb-server-utils software."
                    fi
                fi
                rpm -qa | grep -i '^mariadb-server-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve mariadb-server >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling mariadb-server." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the mariadb-server software."
                    fi
                fi
                rpm -qa | grep -i '^mariadb-errmsg-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve mariadb-errmsg >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling mariadb-errmsg." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the mariadb-errmsg software."
                    fi
                fi
                rpm -qa | grep -i '^perl-DBD-MySQL-[0-9]' >> /dev/null 2>&1
                if [[ $? == 0 ]]; then
                    rpm -ve perl-DBD-MySQL >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling perl-DBD-MySQL." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the DBD-MySQL software."
                    fi
                fi
                if [[ ${DELERROR} == "0" ]]; then
                    rm -fR /var/log/mysql/ /var/lib/mysql /run/mysql/ /usr/share/mariadb/ >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    echo -e "\nThe mariadb software has been successfully removed."
                    echo "[${NOW} install.sh] mariadb has been successfully removed." >> ./${LOGFILE}
                else
                    echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
                    echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
                fi
            fi
        fi
        rpm -qa | grep -i '^neo4j-[0-9]' >> /dev/null 2>&1
        if [[ $? == 0 ]]; then
            if [[ -d "/var/lib/neo4j/data/databases/daccord.db/" ]]; then
                EINGABE=
                while [[ ${EINGABE} != [YyNn] ]]; do
                    read -p $'\033[0;36mWould you like to completely remove the neo4j software as well? [y/n] \033[m' EINGABE
                    if [[ ${EINGABE} != [YyNn] ]]; then
                        echo -e "\n\033[0;35mPlease choose between [y] to completely remove the neo4j software or [n] to skip!\033[m"
                    fi
                done
                if [[ ${EINGABE} == [Yy] ]]; then
                    DELERROR=0
                    echo "[${NOW} install.sh] User has requested to remove neo4j." >> ./${LOGFILE}
                    echo -e "\nPlease wait while the neo4j software is being removed."
                    echo -n '.'
                    systemctl stop neo4j.service >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    systemctl disable neo4j.service >> ./${LOGFILE} 2>&1
                    echo -n '.'
                    rpm -ve neo4j >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling neo4j." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling the neo4j software."
                    fi
                    rpm -ve cypher-shell >> ./${LOGFILE} 2>&1
                    if [[ $? == 0 ]]; then
                        echo -n '.'
                    else
                        DELERROR=1
                        echo "[${NOW} install.sh] Error while uninstalling cypher-shell." >> ./${LOGFILE}
                        echo -e "\nError while uninstalling cypher-shell."
                    fi
                    if [[ ${DELERROR} == "0" ]]; then
                        rm -fR /var/log/neo4j/ /var/lib/neo4j/ /etc/neo4j/ /usr/share/doc/neo4j/ >> ./${LOGFILE} 2>&1
                        echo -n '.'
                        if [[ -e /etc/yum.repos.d/neo4j.repo ]]; then
                            rm -f /etc/yum.repos.d/neo4j.repo >> /dev/null 2>&1
                        fi
                        echo -e "\nThe neo4j software has been successfully removed."
                        echo "[${NOW} install.sh] neo4j has been successfully removed." >> ./${LOGFILE}
                    else
                        echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
                        echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
                    fi
                fi
            fi
        fi
    else
        echo -e "\033[1;31mError while uninstalling some of the components.\033[m"
        echo -e "\033[1;31mTo troubleshoot the installation, please refer to logfile: ${PWD}/${LOGFILE} \033[m"
    fi

}

echo "[${NOW} install.sh] Script started." >> ./${LOGFILE} 2>&1
CHECK_ROOTPRIVILEGE
CHECK_OS
OUTPUT_WELCOME
CHECK_DEV
CHECK_INST_CURRENT
CHECK_PREVINSTALL_FLAG
while true; do
    if [[ ${PREVINSTALL} = 1 ]]; then
        EINGABE=
        while [[ ${EINGABE} != [rRcC] ]]; do
            echo -e "[${NOW} install.sh] A previous unfinished installation has been detected on this system." >> ./${LOGFILE}
            echo -e "\nA previous unfinished installation has been detected on this system."
            read -p $'\033[0;36mWould you like to [r] remove all previously installed components or [c] cancel this installation? [r/c] \033[m' EINGABE
            if [[ ${EINGABE} != [rRcC] ]]; then
                echo -e "\n\033[0;35mPlease choose between [r] to remove or [c] to cancel!\033[m"
            fi
        done
        if [[ ${EINGABE} == [rR] ]]; then
            if [[ ${REDHAT} == "TRUE" ]]; then
                UNINSTALL_REDHAT
                rm /opt/GuH/daccord_mse_install_flag >> /dev/null 2>&1
                EXIT
            else
                UNINSTALL_SUSE
                rm /opt/GuH/daccord_mse_install_flag >> /dev/null 2>&1
                EXIT
            fi
        else
            CANCEL
        fi
    elif [[ ${DACCORD_CURRENT_STAT} = 1 ]]; then
        EINGABE=
        while [[ ${EINGABE} != [uUrRcC] ]]; do
            echo -e "\nThere is already at least one daccord Microsoft Edition component installed on this system."
            read -p $'\033[0;36mWould you like to update [u] the components, remove [r] the components or cancel [c] to exit the script? [u/r/c] \033[m' EINGABE
            if [[ ${EINGABE} != [uUrRcC] ]]; then
                echo -e "\n\033[0;35mPlease choose between update [u] the components, remove [r] the components or cancel [c] to exit the script!\033[m"
            fi
        done
        if [[ ${EINGABE} == [uU] ]]; then
            EINGABE=
            while [[ ${EINGABE} != [OoLl] ]]; do
                echo -e "\nThe install script provides two update modes."
                echo -e "     [o] Online mode - The components will be downloaded from their respective repositories and updated."
                echo -e "     [l] Local mode - The components will be updated from local rpm files. These need to be present in the directory this script is executed from."
                read -p  $'\033[0;36mPlease enter the letter for the desired update mode and confirm with the return key: [o/l] \033[m' EINGABE
                if [[ ${EINGABE} != [OoLl] ]]; then
                    echo -e "\n\033[0;35mPlease choose between [o] for online mode or [l] for local mode!\033[m"
                fi
            done
            if [[ ${EINGABE} == [Oo] ]]; then
                ONLINE_UPDATE
            else
                LOCAL_UPDATE
            fi
        elif [[ ${EINGABE} == [rR] ]]; then
            if [[ ${REDHAT} == "TRUE" ]]; then
                UNINSTALL_REDHAT
                EXIT
            else
                UNINSTALL_SUSE
                EXIT
            fi
        else
            CANCEL
        fi
    else
        EINGABE=
        while [[ ${EINGABE} != [OoLl] ]]; do
            echo -e "\nThe install script provides two installation modes."
            echo -e "     [o] Online mode - The required repositories will be added and the components will be downloaded from their respective repository."
            echo -e "     [l] Local mode - The components will be installed from local rpm files. These need to be present in the directory this script is executed from."
            read -p  $'\033[0;36mPlease enter the letter for the desired installation mode and confirm with the return key: [o/l] \033[m' EINGABE
            if [[ ${EINGABE} != [OoLl] ]]; then
                echo -e "\n\033[0;35mPlease choose between [o] for online mode or [l] for local mode!\033[m"
            fi
        done
        if [[ ${EINGABE} == [Oo] ]]; then
            SET_PREVINSTALL_FLAG
            if [[ ${REDHAT} == "TRUE" ]]; then
                CHECK_MYSQL_REDHAT
                CHECK_NEO4J_REDHAT
            else
                CHECK_MYSQL_SUSE
                CHECK_NEO4J_SUSE
            fi
            ONLINE_INSTALL
            PRESS_ANY_KEY
            exit 1
        else
            SET_PREVINSTALL_FLAG
            if ls ./*.rpm >> /dev/null 2>&1; then
                if [[ ${REDHAT} == "TRUE" ]]; then
                    CHECK_MYSQL_LOCAL_REDHAT
                    CHECK_NEO4J_LOCAL_REDHAT
                else
                    CHECK_MYSQL_LOCAL_SUSE
                    CHECK_NEO4J_LOCAL_SUSE
                fi
                LOCAL_INSTALL
                PRESS_ANY_KEY
                exit 1
            else
                echo -e "\nNo rpm files have been found. Please make sure you have all required rpm files in the same directory as the install script."
                echo -e "[${NOW} install.sh] No rpm files have been found. Please make sure you have all required rpm files in the same directory as the install script." >> ./${LOGFILE}
                CANCEL
            fi
        fi
    fi
done
